@@ -12,8 +12,8 @@ G_END_DECLS
 
 #include <gtksourceview/gtksourceview.h>
 G_BEGIN_DECLS
-#define GTK_TYPE_SOURCE_VIEW_SMART_HOME_END_TYPE gtk_source_view_smart_home_end_type_get_type()
-GType gtk_source_view_smart_home_end_type_get_type (void);
+#define GTK_TYPE_SOURCE_SMART_HOME_END_TYPE gtk_source_smart_home_end_type_get_type()
+GType gtk_source_smart_home_end_type_get_type (void);
 G_END_DECLS
 
 #endif /* __GTKSOURCEVIEW_TYPEBUILTINS_H__ */
